#Metadata
#Aaron Maguire
#28/01/24

#Can't be run as it requires confidential quantative data from users
#Takes in the merged file of all quantative data with codes and themes applied on the end
#Calculates every piece of data required for the analysis and then generates many many pieces of quantative analysis into the console
#Also generates the figures used, 2 and 4 only worked in minimised mode, figure 3 runs only in non minimised mode using data garnered in minimised mode

##Input:
#File containing all codes and themes : file 1
#File containing all quantative data : file 2
#Ideally, this would have just been the large file I made, however some moving of data presented this as the easiest solution

##Output:
#ExampleQuantativeAnalysisPrint.txt
#Figure2-4 as a Matplotlib window. Then saved to png if desired

import openpyxl
import pandas as pd
import numpy as np
import copy

FILE_1_PATH = "ProcessedSessionsData.xlsx"
FILE_2_PATH = "OutputAllCodesAndDataFinal.xlsx"

#i found pandas more useful in this case, rather than openpyxl
df1 = pd.read_excel('OutputAllCodesAndDataFinal.xlsx')

THEMES = ["awareness", "motivations", "environmentalfactors", "positiveaction", "obstacles"]

#Set of PIDs for users with a valid entry
PIDs = set([])
#List of PIDSession tuples for valid entries
PIDsSessions = []
#Set of PIDs for users who didn't drop out
finishPIDs = set([])


validDataPoint = 0

#Dictionary to check the quantity of themes in each competency
themeDict = {"comp1": [0, 0, 0, 0, 0], "comp2": [0, 0, 0, 0, 0], "comp3": [0, 0, 0, 0, 0], "comp4": [0, 0, 0, 0, 0]}

#Dictionary to check the changes in competency utility value when a theme is expressed
themeChanges = {"comp1": [[], [], [], [], []], "comp2": [[], [], [], [], []], "comp3": [[], [], [], [], []], "comp4": [[], [], [], [], []]}


for index, row in df1.iterrows():
    codes = row['Codes'].split()  # Split the string into a list
    sessionNum = row['Unnamed: 1']
    # Check if the list contains an entry and it isn't just 'participation'
    if len(codes) > 0 and codes[0] != '[]' and codes[0] != "['participation']":
        # Grab the value from the "Unnamed: 0" column which is the PID
        PID = row['Unnamed: 0']
        validDataPoint += 1
        PIDs.add(PID)
        PIDsSessions.append([PID, row["Unnamed: 1"]])
    
    if(sessionNum == 6):
        PID = row['Unnamed: 0']
        finishPIDs.add(PID)

#Set of PIDS for 
PIDSforAge = copy.deepcopy(PIDs)
#list of ages
ages = []
#list to get mean and std of ages with a valid response
QuantityofValidAges = []

#awful way of getting the total valid entries by competency. Should have used a list
QuantityofValids1 = 0
QuantityofTotal1 = 0
QuantityofValids2 = 0
QuantityofTotal2 = 0
QuantityofValids3 = 0
QuantityofTotal3 = 0
QuantityofValids4 = 0
QuantityofTotal4 = 0

#minimised mode flips between "valid" (used in the thematic analysis) and all data
#Figures don't always work in non minimised mode
#Ironically, minimised mode has more code but operates on a smaller dataset, hence the name
minimisedMode = True

#list of education levels
education_Levels = ["No formal qualifications", "Secondary education (e.g. GED/GCSE)", "High school diploma/A-levels", "Technical/community college", "Undergraduate degree (BA/BSc/other)", "Graduate degree (MA/MSc/MPhil/other)", "Doctorate degree (PhD/other)", "Don't know / not applicable"]

#list of gender identities in the study
gender_identity = ["Man (including Trans Male/Trans Man)", "Woman (including Trans Female/Trans Woman)", "Non-binary (would like to give more detail)", "Rather not say"]

#list of the persuasive activities I was working with
RelevantClusters = ["Knowledge of how to maintain/achieve mental well-being", "Knowledge of how to maintain/achieve well-being", "Motivation to Change", "Awareness of Smoking Patterns"]

## the following comments illustrate the general data storing technique which is duplicated for each relevant competency

#awful way of getting the average initial change in utility score after an activity has been performed, should have been a list
competency1IChangeTotal = 0
competency1IChangeQuantity = 0
#a value which tracks the highest utility score
competency1HighestChange = 0
#awful way of tracking the total opinion, was turned into a list later
competency1TotalOpinion = 0
#awful way of tracking the valid opinions, was turned into a list later
competency1ValidOptions = 0
#list for tracking opinions of a competency
competency1OpinionsList = []
#awful way of tracking the first opinion for inital comparisons, should have been a list
competency1initialTotalOpinion = 0
competency1initialValidOptions = 0
#I had learnt to use lists at this point, tracks the opinions of people who didn't dropout of the study
competency1opinionNoDropouts = []
#Tracks the education level of valid entries into competency 1 to see if the distribution differed from those who entered the study
educationLevelCounterV1 = [0, 0, 0, 0, 0, 0, 0, 0]
#tracks the quantity of gender identities of valid entries in competency 1
gendQuantityV1 = [0, 0, 0, 0]
#tracks the quantity of ages in valid entries in competency 1
comp1AgeTotal = []


cluster2IChangeTotal = 0
cluster2IChangeQuantity = 0
cluster2HighestChange = 0
competency2TotalOpinion = 0
competency2ValidOptions = 0
competency2OpinionsList = []
competency2initialTotalOpinion = 0
competency2initialValidOptions = 0
competency2opinionNoDropouts = []
educationLevelCounterV2 = [0, 0, 0, 0, 0, 0, 0, 0]
gendQuantityV2 = [0, 0, 0, 0]
comp2AgeTotal = []


cluster3IChangeTotal = 0
cluster3IChangeQuantity = 0
cluster3HighestChange = 0
competency3TotalOpinion = 0
competency3ValidOptions = 0
competency3OpinionsList = []
competency3initialTotalOpinion = 0
competency3initialValidOptions = 0
competency3opinionNoDropouts = []
educationLevelCounterV3 = [0, 0, 0, 0, 0, 0, 0, 0]
gendQuantityV3 = [0, 0, 0, 0]
comp3AgeTotal = []


cluster4IChangeTotal = 0
cluster4IChangeQuantity = 0
cluster4HighestChange = 0
competency4TotalOpinion = 0
competency4ValidOptions = 0
competency4OpinionsList = []
competency4initialTotalOpinion = 0
competency4initialValidOptions = 0
competency4opinionNoDropouts = []
educationLevelCounterV4 = [0, 0, 0, 0, 0, 0, 0, 0]
gendQuantityV4 = [0, 0, 0, 0]
comp4AgeTotal = []

#the columns in which the utilty score for a respective competency was held 
columnComp1 = 15
columnComp2 = 18
columnComp3 = 14
columnComp4 = 17

#track variable for iterating down through the PIDs
currentPID = ""

#the loading the files into openpyxl
file_1 = openpyxl.load_workbook(FILE_1_PATH).active
file_2 = openpyxl.load_workbook(FILE_2_PATH).active

#function for if a theme is spotted. takes in theme column from appropriate row, iterates to see if the string contains the theme, if so appends it to the passed compx in the dict
def themeIterate(compi, info):
    for xi in range(0, 5):
        if THEMES[xi] in info:
            take = themeDict[compi]
            take[xi] = take[xi] + 1
            themeDict[compi] = take

#same as above but for theme changes in utility value. due to this, needs that column too
def themeTrackChange(compi, info, themechecks):
    for xi in range(0, 5):
        if THEMES[xi] in themechecks:
            themeChanges[compi][xi].append(info)

#grabs the file to iterate all the way through it
cnt_row = file_1.max_row
if(minimisedMode):
    for i in range(2, cnt_row):
        grabbedPID = str(file_1.cell(row = i, column = 1).value)
        #grab the current PID for looking at interesting data, checks if we've iterated a PID and if so replaces currentPID
        if(currentPID != grabbedPID and grabbedPID != "None"):
            #append the age, if we've changed PID we've changed person
            ages.append(int(file_2.cell(row = i, column = 26).value))
            currentPID = grabbedPID
        #for competency 1, checks if this isn't an incomplete entry or a post questionnaire (no entry in utility value)
        #this is minimised mode, so also check this is a valid entry
        if (str(file_1.cell(row = i, column = columnComp1).value) != "None" and currentPID in PIDs):
            #append this opinion value to the total 
            competency1TotalOpinion += int(file_1.cell(row = i, column = columnComp1).value)
            competency1OpinionsList.append(int(file_1.cell(row = i, column = columnComp1).value))
            #count this as a valid opinion as it 
            competency1ValidOptions +=1
            #if this is the first session, grab their initial value
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency1initialTotalOpinion += int(file_1.cell(row = i, column = columnComp1).value)
                competency1initialValidOptions +=1
        if (str(file_1.cell(row = i, column = columnComp2).value) != "None"  and currentPID in PIDs):
            competency2TotalOpinion += int(file_1.cell(row = i, column = columnComp2).value)
            competency2OpinionsList.append(int(file_1.cell(row = i, column = columnComp2).value))
            competency2ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency2initialTotalOpinion += int(file_1.cell(row = i, column = columnComp2).value)
                competency2initialValidOptions +=1
        if (str(file_1.cell(row = i, column = columnComp3).value) != "None"  and currentPID in PIDs):
            competency3TotalOpinion += int(file_1.cell(row = i, column = columnComp3).value)
            competency3OpinionsList.append(int(file_1.cell(row = i, column = columnComp3).value))
            competency3ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency3initialTotalOpinion += int(file_1.cell(row = i, column = columnComp3).value)
                competency3initialValidOptions +=1
        if (str(file_1.cell(row = i, column = columnComp4).value) != "None"  and currentPID in PIDs):
            competency4TotalOpinion += int(file_1.cell(row = i, column = columnComp4).value)
            competency4OpinionsList.append(int(file_1.cell(row = i, column = columnComp4).value))
            competency4ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency4initialTotalOpinion += int(file_1.cell(row = i, column = columnComp4).value)
                competency4initialValidOptions +=1

        #if this is competency 1, iterate the amount of competency 1's we have
        if (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[0].lower()):
            QuantityofTotal1 += 1
            #if this is a valid session (user completed and had a response the next session)
            if([currentPID, int(file_1.cell(row = i+1, column = 2).value)] in PIDsSessions):
                #append a single instance of this users age to check how many people with ages did a valid response for distro, then discard so no duplicates
                if(currentPID in PIDSforAge):
                    QuantityofValidAges.append(int(file_2.cell(row = i, column = 26).value))
                    PIDSforAge.discard(currentPID)
                #this is a valid session, grab their genderI, education level and append their age (not relevant due to previous check)
                gendQuantityV1[gender_identity.index(str(file_2.cell(row = i, column = 24).value))] += 1
                educationLevelCounterV1[education_Levels.index(str(file_2.cell(row = i, column = 25).value))] += 1
                comp1AgeTotal.append(int(file_2.cell(row = i, column = 26).value))
                #check for themes present in this valid entry, if they appear, iterate them in the dictionary using the function if so
                themeIterate("comp1", str(file_2.cell(row = i+1, column = 22).value))
                #valid entry, so iterate quantity
                QuantityofValids1+=1
            #if the user has a session entry next, that session isn't incomplete or a post questionnaire
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp1).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                themeTrackChange("comp1", int(int(file_2.cell(row = i+1, column = columnComp1).value) - int(file_2.cell(row = i, column = columnComp1).value)), str(file_2.cell(row = i+1, column = 22).value))
                #mark the change in their opinion of the actvivity
                change = int(file_1.cell(row = i+1, column = columnComp1).value) - int(file_1.cell(row = i, column = columnComp1).value)
                #add this change to the total
                competency1IChangeTotal += change
                #mark a valid datapoint for analysis
                competency1IChangeQuantity+=1
                #keep track of the biggest change for curiousity's sake
                if(change > competency1HighestChange):
                    competency1HighestChange = change
        #it's not competency 1, so elif comp 2
        elif (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[1].lower()):
            QuantityofTotal2 += 1
            if([currentPID, int(file_1.cell(row = i+1, column = 2).value)] in PIDsSessions):
                if(currentPID in PIDSforAge):
                    QuantityofValidAges.append(int(file_2.cell(row = i, column = 26).value))
                    PIDSforAge.discard(currentPID)
                gendQuantityV2[gender_identity.index(str(file_2.cell(row = i, column = 24).value))] += 1
                educationLevelCounterV2[education_Levels.index(str(file_2.cell(row = i, column = 25).value))] += 1
                comp2AgeTotal.append(int(file_2.cell(row = i, column = 26).value))
                themeIterate("comp2", str(file_2.cell(row = i+1, column = 22).value))
                QuantityofValids2+=1
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp2).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                themeTrackChange("comp2", int(int(file_2.cell(row = i+1, column = columnComp2).value) - int(file_2.cell(row = i, column = columnComp2).value)), str(file_2.cell(row = i+1, column = 22).value))
                change = int(file_1.cell(row = i+1, column = columnComp2).value) - int(file_1.cell(row = i, column = columnComp2).value)
                cluster2IChangeTotal += change
                cluster2IChangeQuantity+=1
                if(change > cluster2HighestChange):
                    cluster2HighestChange = change

        #it's not competency 2, so elif comp 3
        elif (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[2].lower()):
            QuantityofTotal3 += 1
            if([currentPID, int(file_1.cell(row = i+1, column = 2).value)] in PIDsSessions):
                if(currentPID in PIDSforAge):
                    QuantityofValidAges.append(int(file_2.cell(row = i, column = 26).value))
                    PIDSforAge.discard(currentPID)
                gendQuantityV3[gender_identity.index(str(file_2.cell(row = i, column = 24).value))] += 1
                educationLevelCounterV3[education_Levels.index(str(file_2.cell(row = i, column = 25).value))] += 1
                comp3AgeTotal.append(int(file_2.cell(row = i, column = 26).value))
                themeIterate("comp3", str(file_2.cell(row = i+1, column = 22).value))
                QuantityofValids3+=1
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp3).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                change = int(file_1.cell(row = i+1, column = columnComp3).value) - int(file_1.cell(row = i, column = columnComp3).value)
                themeTrackChange("comp3", int(int(file_2.cell(row = i+1, column = columnComp3).value) - int(file_2.cell(row = i, column = columnComp3).value)), str(file_2.cell(row = i+1, column = 22).value))
                cluster3IChangeTotal += change
                cluster3IChangeQuantity+=1
                if(change > cluster3HighestChange):
                    cluster3HighestChange = change


        #it's not competency 3, so elif comp 4
        elif (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[3].lower()):
            QuantityofTotal4 += 1
            if([currentPID, int(file_1.cell(row = i+1, column = 2).value)] in PIDsSessions):
                if(currentPID in PIDSforAge):
                    QuantityofValidAges.append(int(file_2.cell(row = i, column = 26).value))
                    PIDSforAge.discard(currentPID)
                gendQuantityV4[gender_identity.index(str(file_2.cell(row = i, column = 24).value))] += 1
                educationLevelCounterV4[education_Levels.index(str(file_2.cell(row = i, column = 25).value))] += 1
                comp4AgeTotal.append(int(file_2.cell(row = i, column = 26).value))
                themeIterate("comp4", str(file_2.cell(row = i+1, column = 22).value))
                QuantityofValids4+=1
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp4).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                change = int(file_1.cell(row = i+1, column = columnComp4).value) - int(file_1.cell(row = i, column = columnComp4).value)
                themeTrackChange("comp4", int(int(file_2.cell(row = i+1, column = columnComp4).value) - int(file_2.cell(row = i, column = columnComp4).value)), str(file_2.cell(row = i+1, column = 22).value))
                cluster4IChangeTotal += change
                cluster4IChangeQuantity+=1
                if(change > cluster4HighestChange):
                    cluster4HighestChange = change

#non minimised mode is the exact same as minimised mode except there is no check for validity in the thematic analysis. this is the whole dataset
else:
    for i in range(2, cnt_row):
        grabbedPID = str(file_1.cell(row = i, column = 1).value)
        #grab the current PID for looking at interesting data
        if(currentPID != grabbedPID and grabbedPID != "None"):
            currentPID = grabbedPID

        if (str(file_1.cell(row = i, column = columnComp1).value) != "None"):
            competency1TotalOpinion += int(file_1.cell(row = i, column = columnComp1).value)
            competency1OpinionsList.append(int(file_1.cell(row = i, column = columnComp1).value))
            if(grabbedPID in finishPIDs):
                competency1opinionNoDropouts.append(int(file_1.cell(row = i, column = columnComp1).value))
            competency1ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency1initialTotalOpinion += int(file_1.cell(row = i, column = columnComp1).value)
                competency1initialValidOptions +=1
        if (str(file_1.cell(row = i, column = columnComp2).value) != "None"):
            competency2TotalOpinion += int(file_1.cell(row = i, column = columnComp2).value)
            competency2OpinionsList.append(int(file_1.cell(row = i, column = columnComp2).value))
            if(grabbedPID in finishPIDs):
                competency2opinionNoDropouts.append(int(file_1.cell(row = i, column = columnComp2).value))
            competency2ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency2initialTotalOpinion += int(file_1.cell(row = i, column = columnComp2).value)
                competency2initialValidOptions +=1
        if (str(file_1.cell(row = i, column = columnComp3).value) != "None"):
            competency3TotalOpinion += int(file_1.cell(row = i, column = columnComp3).value)
            competency3OpinionsList.append(int(file_1.cell(row = i, column = columnComp3).value))
            if(grabbedPID in finishPIDs):
                competency3opinionNoDropouts.append(int(file_1.cell(row = i, column = columnComp3).value))
            competency3ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency3initialTotalOpinion += int(file_1.cell(row = i, column = columnComp3).value)
                competency3initialValidOptions +=1
        if (str(file_1.cell(row = i, column = columnComp4).value) != "None"):
            competency4TotalOpinion += int(file_1.cell(row = i, column = columnComp4).value)
            competency4OpinionsList.append(int(file_1.cell(row = i, column = columnComp4).value))
            if(grabbedPID in finishPIDs):
                competency4opinionNoDropouts.append(int(file_1.cell(row = i, column = columnComp4).value))
            competency4ValidOptions +=1
            if (str(file_1.cell(row = i, column = 2).value) == "1"):
                competency4initialTotalOpinion += int(file_1.cell(row = i, column = columnComp4).value)
                competency4initialValidOptions +=1

        if (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[0].lower()):
            #if the user has a session entry next, that session isn't incomplete or a post questionnaire
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp1).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                #mark the change in their opinion of the actvivity
                change = int(file_1.cell(row = i+1, column = columnComp1).value) - int(file_1.cell(row = i, column = columnComp1).value)
                #add this change to the total
                competency1IChangeTotal += change
                #mark a valid datapoint for analysis
                competency1IChangeQuantity+=1
                #keep track of the biggest change for curiousity's sake
                if(change > competency1HighestChange):
                    competency1HighestChange = change

        elif (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[1].lower()):
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp2).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                change = int(file_1.cell(row = i+1, column = columnComp2).value) - int(file_1.cell(row = i, column = columnComp2).value)
                cluster2IChangeTotal += change
                cluster2IChangeQuantity+=1
                if(change > cluster2HighestChange):
                    cluster2HighestChange = change

        elif (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[2].lower()):
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp3).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                change = int(file_1.cell(row = i+1, column = columnComp3).value) - int(file_1.cell(row = i, column = columnComp3).value)
                cluster3IChangeTotal += change
                cluster3IChangeQuantity+=1
                if(change > cluster3HighestChange):
                    cluster3HighestChange = change

        elif (str(file_1.cell(row = i, column = 6).value).lower() == RelevantClusters[3].lower()):
            if(str(file_1.cell(row = i+1, column = 1).value) == "None" and str(file_1.cell(row = i+1, column = columnComp4).value) != "None" and int(file_1.cell(row = i+1, column = 2).value) != 6):
                change = int(file_1.cell(row = i+1, column = columnComp4).value) - int(file_1.cell(row = i, column = columnComp4).value)
                cluster4IChangeTotal += change
                cluster4IChangeQuantity+=1
                if(change > cluster4HighestChange):
                    cluster4HighestChange = change
        
print("Total quantity of valid data points:" + str(validDataPoint))

avgChange1 = competency1IChangeTotal/competency1IChangeQuantity
avgChange2 = cluster2IChangeTotal/cluster2IChangeQuantity
avgChange3 = cluster3IChangeTotal/cluster3IChangeQuantity
avgChange4 = cluster4IChangeTotal/cluster4IChangeQuantity


print("Average Immediate Change in Persuasive activity " + RelevantClusters[0] + ": " + str(avgChange1))
print("Average Immediate Change in Persuasive activity " + RelevantClusters[1] + ": " + str(avgChange2))
print("Average Immediate Change in Persuasive activity " + RelevantClusters[2] + ": " + str(avgChange3))
print("Average Immediate Change in Persuasive activity " + RelevantClusters[3] + ": " + str(avgChange4))


print("Average Immediate Change in Persuasive activity 1 " + str(np.mean(competency1OpinionsList)))
print("Average Immediate Change in Persuasive activity 2 " + str(np.mean(competency2OpinionsList)))
print("Average Immediate Change in Persuasive activity 3 " + str(np.mean(competency3OpinionsList)))
print("Average Immediate Change in Persuasive activity 4 " + str(np.mean(competency4OpinionsList)))


print("Average STD in Persuasive activity 1 " + str(np.std(competency1OpinionsList)))
print("Average STD in Persuasive activity 2 " + str(np.std(competency2OpinionsList)))
print("Average STD in Persuasive activity 3 " + str(np.std(competency3OpinionsList)))
print("Average STD in Persuasive activity 4 " + str(np.std(competency4OpinionsList)))

print("Quantity of activity 1 results: " + str(competency1IChangeQuantity))
print("Quantity of activity 2 results: " + str(cluster2IChangeQuantity))
print("Quantity of activity 3 results: " + str(cluster3IChangeQuantity))
print("Quantity of activity 4 results: " + str(cluster4IChangeQuantity))


print("Highest change of activity 1 results: " + str(competency1HighestChange))
print("Highest change of activity 2 results: " + str(cluster2HighestChange))
print("Highest change of activity 3 results: " + str(cluster3HighestChange))
print("Highest change of activity 4 results: " + str(cluster4HighestChange))

print("Quantity of Valids 1:" + str(QuantityofValids1))
print("Quantity of Valids 2:" + str(QuantityofValids2))
print("Quantity of Valids 3:" + str(QuantityofValids3))
print("Quantity of Valids 4:" + str(QuantityofValids4))

print("Average opinion of competency 1: " + str(competency1TotalOpinion/competency1ValidOptions))
print("Average opinion of competency 2: " + str(competency2TotalOpinion/competency2ValidOptions))
print("Average opinion of competency 3: " + str(competency3TotalOpinion/competency3ValidOptions))
print("Average opinion of competency 4: " + str(competency4TotalOpinion/competency4ValidOptions))

print("Average initial opinion of competency 1: " + str(competency1initialTotalOpinion/competency1initialValidOptions))
print("Average initial opinion of competency 2: " + str(competency2initialTotalOpinion/competency2initialValidOptions))
print("Average initial opinion of competency 3: " + str(competency3initialTotalOpinion/competency3initialValidOptions))
print("Average initial opinion of competency 4: " + str(competency4initialTotalOpinion/competency4initialValidOptions))

print("Education Distribution of engaged Comp 1:")
print(educationLevelCounterV1)
print("Education Distribution of engaged Comp 2:")
print(educationLevelCounterV2)
print("Education Distribution of engaged Comp 3:")
print(educationLevelCounterV3)
print("Education Distribution of engaged Comp 4:")
print(educationLevelCounterV4)

totalAgeMean = np.mean(QuantityofValidAges)
print(len(QuantityofValidAges))
print("Useful Ages:" + str(totalAgeMean))
print("Age STD:" + str(np.std(QuantityofValidAges)))
print("Didn't dropout: " + str(len(finishPIDs)))

print("Quantity of themes in competencies: ")
print(themeDict)

print("Average Change of Awareness in Competency Value: " + str(np.mean(themeChanges["comp1"][0] + themeChanges["comp2"][0] + themeChanges["comp3"][0] + themeChanges["comp4"][0])))
print("Average Change of Motivation in Competency Value: " + str(np.mean(themeChanges["comp1"][1] + themeChanges["comp2"][1] + themeChanges["comp3"][1] + themeChanges["comp4"][1])))
print("Average Change of Environmental Factors in Competency Value: " + str(np.mean(themeChanges["comp1"][2] + themeChanges["comp2"][2] + themeChanges["comp3"][2] + themeChanges["comp4"][2])))
print("Average Change of Positive Action in Competency Value: " + str(np.mean(themeChanges["comp1"][3] + themeChanges["comp2"][3] + themeChanges["comp3"][3] + themeChanges["comp4"][3])))
print("Average Change of Obstacles in Competency Value: " + str(np.mean(themeChanges["comp1"][4] + themeChanges["comp2"][4] + themeChanges["comp3"][4] + themeChanges["comp4"][4])))

print("Average change of Motivation in activity 3:" + str(np.mean(themeChanges["comp2"][3])))

print(np.std(themeChanges["comp1"][4] + themeChanges["comp2"][4] + themeChanges["comp3"][4] + themeChanges["comp4"][4]))
print(len(themeChanges["comp1"][4] + themeChanges["comp2"][4] + themeChanges["comp3"][4] + themeChanges["comp4"][4]))

print("Average Immediate Change: " + str((avgChange1+avgChange2+avgChange3+avgChange4)/4))

labels = ("Competency 1", "Competency 2", "Competency 3", "Competency 4",)

#hardcoded values from running in minimised mode, figure 3 is only plotted when not in minimised mode
values = {'Initial Opinion (Session 1)' : ((competency1initialTotalOpinion/competency1initialValidOptions), (competency2initialTotalOpinion/competency2initialValidOptions), (competency3initialTotalOpinion/competency3initialValidOptions), (competency4initialTotalOpinion/competency4initialValidOptions)), 
          'Average Opinion over Study' : ((competency1TotalOpinion/competency1ValidOptions), (competency2TotalOpinion/competency2ValidOptions), (competency3TotalOpinion/competency3ValidOptions), (competency4TotalOpinion/competency4ValidOptions)) , 'Average Opinion over Study, Dropouts Excluded' : (np.mean(competency1opinionNoDropouts), np.mean(competency2opinionNoDropouts), np.mean(competency3opinionNoDropouts), np.mean(competency4opinionNoDropouts)),
          'Average Opinion over Study with Valid Response': (6.086281276962899, 5.821243523316062, 5.673856773080241, 4.616580310880829)}

##Figure 3

# x = np.arange(len(labels))  # the label locations
# width = 0.23  # the width of the bars
# multiplier = 0

# fig, ax = plt.subplots()

# for attribute, measurement in values.items():
#     offset = width * multiplier
#     rects = ax.bar(x + offset, measurement, width, label=attribute)
#     roundedMeas = []
#     for i in measurement:
#         roundedMeas.append(round(i, 2))
#     ax.bar_label(rects, labels=tuple(roundedMeas), padding=3, fontsize=8)
#     multiplier += 1

# # Add some text for labels, title and custom x-axis tick labels, etc.
# ax.set_ylabel('Competency Score (-10 to 10)')
# ax.set_title('Competency Scores Throughout the Study')
# ax.set_xticks(x + (width*1.5), labels)
# ax.legend(loc='best')
# ax.set_ylim(0, 10)

# plt.show()

##Figure 2

# counter = Counter(ages)

# # Extract unique elements and their counts
# unique_elements = list(counter.keys())
# counts = list(counter.values())

# counterV = Counter(QuantityofValidAges)

# # Extract unique elements and their counts
# unique_elementsV = list(counterV.keys())
# countsV = list(counterV.values())

# # Create a bar chart
# plt.bar(unique_elements, counts, color='skyblue', label='Total Participants')
# plt.bar(unique_elementsV, countsV, color='red', label='Valid Data Participants')


# # Add labels and title
# plt.xlabel('Age')
# plt.ylabel('Occurances')
# plt.title('Age of Participants')

# # Show the plot
# plt.legend(loc = 'best ')
# plt.show()


##Figure X, unused

# data = {"1" : [QuantityofValids1, QuantityofTotal1-QuantityofValids1], "2" :[QuantityofValids2, QuantityofTotal2-QuantityofValids2], "3" :[QuantityofValids3, QuantityofTotal3-QuantityofValids3], "4" :[QuantityofValids4, QuantityofTotal4-QuantityofValids4]}
# category_names = ["Valid Responses", "'None' or Invalid Responses"]
# #source https://matplotlib.org/3.1.1/gallery/lines_bars_and_markers/horizontal_barchart_distribution.html#sphx-glr-gallery-lines-bars-and-markers-horizontal-barchart-distribution-py
# def survey(results, category_names):
#     labels = list(results.keys())
#     data = np.array(list(results.values()))
#     data_cum = data.cumsum(axis=1)
#     category_colors = plt.get_cmap('RdYlGn')(
#         np.linspace(0.15, 0.85, data.shape[1]))

#     fig, ax = plt.subplots(figsize=(9.2, 5))
#     ax.invert_yaxis()
#     ax.xaxis.set_visible(False)
#     ax.set_xlim(0, np.sum(data, axis=1).max())

#     for i, (colname, color) in enumerate(zip(category_names, category_colors)):
#         widths = data[:, i]
#         starts = data_cum[:, i] - widths
#         ax.barh(labels, widths, left=starts, height=0.5,
#                 label=colname, color=color)
#         xcenters = starts + widths / 2

#         r, g, b, _ = color
#         text_color = 'white' if r * g * b < 0.5 else 'darkgrey'
#         for y, (x, c) in enumerate(zip(xcenters, widths)):
#             ax.text(x, y, str(int(c)), ha='center', va='center',
#                     color=text_color)
#     ax.legend(ncol=len(category_names), bbox_to_anchor=(0, 1),
#               loc='lower left', fontsize='small')

#     return fig, ax

# survey(data, category_names)
# plt.xlabel("Quantity of Answers")
# plt.ylabel("Competency")
# plt.show()

##Figure 4


# Labels = ["\nAwareness", "Motivations", "\nEnvironmental Factors", "Positive Action", "\nObstacles"]
# Competency = ["Competency 1", "Competency 2", "Competency 3", "Competency 4"]
# # Extract keys and values from the dictionary
# categories = list(themeDict.keys())
# values = list(themeDict.values())

# # Determine the number of bars for each category
# num_bars = len(categories[0])

# # Set the width of the bars
# bar_width = 0.2

# # Create a bar chart
# fig, ax = plt.subplots()

# for i, category in enumerate(Competency):
#     # Calculate the x-position for each bar
#     x_positions = [j + i * bar_width for j in range(num_bars)]
    
#     # Plot the bars for each category
#     ax.bar(x_positions, values[i], width=bar_width, label=category)

# # Set labels and title
# ax.set_ylabel('Quantity')
# ax.set_title('Theme Quantity through Competencies')

# # Set x-axis ticks and labels
# ax.set_xticks([j + (len(categories) - 1) * bar_width / 2 for j in range(num_bars)])
# ax.set_xticklabels([Labels[j] for j in range(num_bars)], fontsize = 8)

# # Show legend
# ax.legend()

# # Show the plot
# plt.show()