#Metadata
#Aaron Maguire
#28/01/24


#file which can't be run as it requires the codes sorted alphabetically based on confidential data
#purpose was to check the cohens coefficient of each code, determine the total cohens coefficient
#and then glean the brennan prediger coefficient of each code, printing them all to console

##Inputs: 
#my codes, sorted alphabetically on user response:  "file_sorted_and_cleaned.xlsx"
#peer codes, sorted alphabetically on user response: "peerfile_sorted_and_cleaned.xlsx"

##Outputs:
#Printed to console: CohensKappa.txt

import openpyxl
import re
import numpy as np

#initial loading of the file paths
#my codes, sorted alphabetically on user response
FILE_1_PATH = "file_sorted_and_cleaned.xlsx"
#peer codes, sorted alphabetically on user response
FILE_2_PATH = "peerfile_sorted_and_cleaned.xlsx"


#all the code relevant
UNIQUE_CODES1 = ['Cravings as a Concept', 'Participation', 'Skepticism', 'Exercise Habits', 'Distractions', 'Smoking Habits', 'Negative Awareness', 'Stress', 
                 'Activity Wasn\'t a Priority', 'Sleep Habits', 'Mindfulness', 
                 'Planning Ahead', 'Quantifying Blockers', 
                 'Prioritising Quitting', 
                 'Outside and Inside', 'Proximity to Other Smokers', 
                 'Self Affirmation', 'Links with other Addictions', 'Perception from Loved Ones', 'Discouraged',
                'Intentional Reduction', 'Money as a Factor', 'Inspired Research', 
                'Optimism', 'Future Worries', 'Apathy', 'Perception of Environment']

UNIQUE_CODES = list(map(lambda x: x.lower().replace(' ', ''), UNIQUE_CODES1))

# Class that encapsulates the data of a Quotation
class Quotation:
    def __init__(self, content, codes):
        if content is  None: # in case a None value got into the dataset
            self.content = ''
        else:
            self.content = content
        self.codes = codes.split('\n')
        if len(self.codes) == 1:
            self.codes = self.codes[0].split(',') # Split the codes into a list

    def __eq__(self, __value: object) -> bool:
        if isinstance(__value, Quotation):
            return self.modified_content() == __value.modified_content()
        return False
    
    def __hash__(self):
        return hash(self.modified_content())
    
    # Method that transforms the content string s.t. each quotation is uniquely identifiable
    def modified_content(self):
        modified_content = re.sub(r'\s+', ' ', self.content.strip().lower()) # Remove duplicate spaces and newline characters
        return modified_content[:min(len(modified_content), 30)] # Keep only the first 30 characters (takes into account different column widths from different PDF files)


# Method that creates a coding matrix given a list of Quotations
# - a coding matrix is a matrix that has quotations on its rows and code frequencies on the columns
def create_coding_matrix(quotations, fileNum):

    # Create the coding matrix
    coding_matrix = np.zeros((len(quotations), len(UNIQUE_CODES)), dtype=int)
    for i, quotation in enumerate(quotations):
        for j, code in enumerate(UNIQUE_CODES):
            # If a code appears in a quotation, update its corresponding column index in the matrix to 1
                if code in quotation.codes:
                    coding_matrix[i, j] = 1
    
    return coding_matrix



if __name__ == "__main__":
    # Retrieve the active sheet from each of the 2 files
    file_1 = openpyxl.load_workbook(FILE_1_PATH).active
    file_2 = openpyxl.load_workbook(FILE_2_PATH).active

    # Iterate through all the rows to initialize the Quotation lists for each file
    cnt_row = file_1.max_row
    quotations_1, quotations_2 = list(), list()
    for i in range(1, cnt_row + 1):
        # Get the data from the current row of file 1
        content_1 = str(file_1.cell(row = i, column = 3).value).lower().replace(' ', '')
        codes_1 = str(file_1.cell(row = i, column = 7).value).lower().replace(' ', '')
        quotation_1 = Quotation(content_1, codes_1) # Cast the retrieved data to a Quotation

        # Do the same for file 2
        content_2 = str(file_2.cell(row = i, column = 4).value).lower().replace(' ', '')
        codes_2 = str(file_2.cell(row = i, column = 19).value).lower().replace(' ', '')
        quotation_2 = Quotation(content_2, codes_2)

        # Add the Quotations to their respective lists
        quotations_1.append(quotation_1)
        quotations_2.append(quotation_2)

# Create coding matrices for the 2 quotation lists
coding_matrix_1 = np.nan_to_num(create_coding_matrix(quotations_1, 1))
coding_matrix_2 = np.nan_to_num(create_coding_matrix(quotations_2, 2))

totalcohen = 0

N = len(coding_matrix_1)

#hand coded cohens kappa check for each code
for i in range(coding_matrix_1.shape[1]):  # Iterate over the columns (labels)
    label1 = coding_matrix_1[:, i]
    label2 = coding_matrix_2[:, i]

    TP = 0
    TN = 0
    FN = 0
    FP = 0

    for y in range(len(label1)):
        if(label1[y] == 1 and label2[y] == 1):
            TP+=1
        elif (label1[y] == 0 and label2[y] == 0):
            TN+=1
        elif (label1[y] == 1 and label2[y] == 0):
            FP+=1
        elif(label1[y] == 0 and label2[y] == 1):
            FN+=1
    
    kappa = (2*((TP*TN)-(FN*FP)))/(((TP+FP)*(FP+TN))+((TP+FN)*(FN+TN)))
    totalcohen+=kappa
    print(f"Cohen's kappa for label {i+1}: {kappa}")

#hand coded Brennand Prediger calculation over the dataset
BPResult = (((2*(TP+TN)) - ((TP+TN)+FP)) / (TP+FP+TN+FN))

print("Brennan Prediger = " + str(BPResult))
print(totalcohen/len(UNIQUE_CODES1))