# Examining the Efficacy of Persuasive eHealth Applications in Facilitating Smoking Cessation: An Analysis of Competency Based Activities

Author: Aaron Maguire
Date: 28th January 2024

The following repository contains the data analysis code and confidential data used for the analysis of persuasive activities in persuading users of the utility of competencies in an eHealth application. The dataset of this study is currently confidential and pending review, however when it is posted it will be available through the following osf registry: https://www.osf.io/nuy4w 
Additionally, the competencies used for the study are also under review and will be available here whenever resolved https://osf.io/cax6f/

This work is part of the multidisciplinary research project Perfect Fit, which is supported by several funders organized by the Netherlands Organization for Scientific Research (NWO), program Commit2Data - Big Data \& Health (project number 628.011.211). Besides NWO, the funders include the Netherlands Organisation for Health Research and Development (ZonMw), Hartstichting, the Ministry of Health, Welfare, and Sport (VWS), Health Holland, and the Netherlands eScience Center. 

## Contents of the repository
```
FinalCodeAndData
│
└───Code
│   │   QuantativePreData.py
│   │   QuantativeAnalysis.py
│   │   cohens.py
│   │   Cleanup.py
│   │   PairingCode.py
│
└───FinalData
│   │   FinalCodesAnonymised.xlsx
│   │   ExampleQuantativeAnalysisPrint.txt
│   │   Code Dictionary: Post Review.Docx
│   │   CohensKappa.txt
│
└───Results
│   │   Figure1.png
│   │   Figure2.png
│   │   Figure3.png
│   │	Figure4.png
│
│   README.md
```

| File/Folder name                        | Content                                                      |
|-----------------------------------------|--------------------------------------------------------------|
| Readme.md                               | Current file that you are reading.                           |
| Code/PairingCode.py                     | A file used to pair codes or themes onto session data and PID files. Was used to create the file which is used in QuantativeAnalysis.py |
| Code/Cleanup.py                         | A helper file which changed all codes from an outdated format into a usable format and appended them onto an xlsx file |
| Code/QuantativePreData.py               | File for grabbing the data from prolific_profic_anonym.xlsx (confidential) and appending it to sessions from sessionsdata_anoym.xlsx (confidential) to perform checks with codes and themes  <br> |
| Code/Cohens.py                          | Generates the cohens coefficient for pair coding and the brennan prediger coefficient. Can’t be run as the codes were sorted alphabetically based on the confidential user responses |
| Code/QuantativeAnalysis.py              | Takes in the merged file of all quantative data (confidential) with codes and themes applied on the end. Then calculates every piece of data required for the analysis and then generates many many pieces of quantative analysis into the console Also generates the figures used, 2 and 4 only worked in minimised mode, figure 3 runs only in non minimised mode using data garnered in minimised mode |
| FinalData/FinalCodesAnonymised.xlsx     | A file containing the original codes, peer codes and final codes of each session (sorted by rand_id then session number). Themes determined are also appended.<br>This is the file used in Quantative Analysis but all the confidential data has been removed. |
| FinalData/ExampleQuantativeAnalysis.txt | An example of what is printed in the console from an operational version of QuantativeAnalysis.py  |
| FinalData/CohensKappa.txt               | An example of what is printed in the console from an operational version of Cohens.py |
| FinalData/CodeDictionaryPostReview.docx | The code dictionary used for the final paper                 |
| Figure/Figure1                          | Thematic map used in the final paper                         |
| Figure/Figure2                          | Bar chart showing the utility values of competencies in various states, generated from QuantativeAnalysis.py |
| Figure/Figure3                          | Bar chart showing the age distribution of users alongside how many provided valid data, generated from QuantativeAnalysis.py |
| Figure/Figure4                          | Bar chart showing the occurance of themes in competencies, generated from QuantativeAnalysis.py |

## Explanation of Final Codes Anonymised.xlsx
| Column         | Data                                                         |
|----------------|--------------------------------------------------------------|
| PID            | The user’s rand_id to track their identity in the dataset (when published) |
| Session Number | The session which is being evaluated (corresponds to sessionsdata_anonym.csv in the dataset) |
| FinalCodes     | The final codes present after review and reworking the code dictionary |
| Themes         | The final themes present at the time of the study            |
| OriginalCodes  | The original codes present, which were then reviewed and refined |
| PeerCodes      | The peer codes, with the code dictionary refinements applied |
